/*
 * dstemp.c
 *
 *  Created on: 9 mar 2017
 *      Author: dsosnowski
 */

#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include "defs.h"
#include "dstemp.h"


/*#define ONEW_PORT			PORTB
#define ONEW_DDR			DDRB
#define ONEW_PIN			PINB
#define ONEW				2
#define ONEW_H				(ONEW_PORT |= (1<<ONEW))
#define ONEW_L				(ONEW_PORT &= ~(1<<ONEW))
#define ONEW_SETO			(ONEW_DDR |= (1<<ONEW))
#define ONEW_SETI			(ONEW_DDR &= ~(1<<ONEW))
#define ONEW_GET			(ONEW_PIN & (1<<ONEW))
*/
/*#define ONEW_PORT			PORTD
#define ONEW_DDR			DDRD
#define ONEW_IN				PIND
#define DSPIN_CH1			7
*/
#define ONEW_L				(ONEW_PORT &= ~(ds.pins[ds.channel]))
#define ONEW_H				(ONEW_PORT |= (ds.pins[ds.channel]))
#define ONEW_SETO			(ONEW_DDR |= (ds.pins[ds.channel]))
#define ONEW_SETI			(ONEW_DDR &= ~(ds.pins[ds.channel]))
#define ONEW_GET			(ONEW_IN & (ds.pins[ds.channel]))

//ds.step
//#define DS_IDLE				0
//#define DS_CONVERTING		1

/*//ds.status
#define DS_OK				0
#define DS_READY			1
#define DS_ERR				2
#define DS_SHORT			3
#define DS_NO_CONNECT		4
*/

#define DS_READ_ROM			0x33
#define DS_SKIP_ROM			0xCC
#define DS_START_CONV		0x44
#define DS_RAED_RESULT		0xBE

#define DS_TYPE_NOREAD		0
#define	DS_TYPE_18B20		0x28
#define	DS_TYPE_18S20		0x10


struct {
	uint8_t step;
	uint8_t channel;
	uint8_t pins[DS_CHANNELS];
	uint8_t type[DS_CHANNELS];
	uint8_t status[DS_CHANNELS];
	uint8_t ecounter[DS_CHANNELS];
	int16_t temperature[DS_CHANNELS];
} ds;


//----------------------------------------
void DsInit(void)
{
	ds.pins[0] = (1<<DSPIN_CH1);
	ds.step = DS_CONVERTING;
	ds.ecounter[0] = DS_ERR_MAX;
}

//----------------------------------------
void DsDelay(uint8_t us)
{
	//TCCR0B = 0;
	//TCCR0A = 0;
	SFIOR = (1<<PSR10);
	TIFR |= (1<<TOV0);
	
	us -= 1;
	TCNT0 = (0xFF - us);
	TCCR0 = (1<<CS01);
	
	while((TIFR & (1<<TOV0)) == 0);
	
	/*
	do 
	{
		//_delay_us(1);
		
		//for 8MHz
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");
		//asm("nop");
		//asm("nop");
		
	} while(--us != 0);
	*/
}

//-------------------------------------------------------
int16_t DsTempGet(uint8_t channel)
{
	return ds.temperature[channel];
}

//-------------------------------------------------------
uint8_t DsStatusGet(uint8_t channel) {
	if (ds.ecounter[channel] >= DS_ERR_MAX) {
		return DS_ERR;
	}
	
	if (ds.status[channel] == DS_READY){
		ds.status[channel] = DS_OK;
		return DS_READY;
	}
	
	return DS_OK;
}

//-------------------------------------------------------
uint8_t DsProcessNext(void)
{
	ds.step++;
	if (ds.step > DS_CONVERTING) ds.step = 0;
	return ds.step;
}

//------------------------------------
//must be execute about 1/1s
uint8_t DsProcess(uint8_t channel)
{
	ds.channel = channel;
	//--------------------------------------------------
	if (ds.step <= DS_IDLE)
	{
		if (DsReset(channel) == DS_OK)
		{
			DsGetType(channel);
			if (ds.status[channel] == DS_OK)
			{
				if (DsReset(channel) == DS_OK)
				{
					DsWriteByte(channel, DS_SKIP_ROM);		//skip ROM
					DsWriteByte(channel, DS_START_CONV);		//convert
					//power for convert
					ONEW_H;
					ONEW_SETO;
				}
			}
		}
	}

	//--------------------------------------------------
	if (ds.step >= DS_CONVERTING)
	{
		if (ds.status[channel] == DS_OK)
		{
			if (DsReset(channel) == DS_OK)
			{
				DsWriteByte(channel, DS_SKIP_ROM);		//skip ROM
				DsWriteByte(channel, DS_RAED_RESULT);		//read
				ds.status[ds.channel] = DsTempCalc(DsReadByte(channel) | (DsReadByte(channel)<<8));
			}
		}
	}

	//--------------------------------------------------
	if (ds.status[ds.channel] == DS_READY){
		ds.ecounter[ds.channel] = 0;
	} else {		
		if (ds.ecounter[ds.channel] < 0xFF)
		ds.ecounter[ds.channel]++;
	}

	//--------------------------------------------------
	return ds.step;
}

//-------------------------------------
uint8_t DsTempCalc(int16_t data) {
	int32_t buffer = 0;

	if (ds.type[ds.channel] == DS_TYPE_18B20) {
		buffer = (int32_t)data;
		buffer *= 625;
		buffer += 555;						/////////////////??????????????????????????????????
		buffer /= 1000;
	} else if (ds.type[ds.channel] == DS_TYPE_18S20) {
		buffer = (data * 5);
	} else {
		buffer = 10000;
	}

	if ((buffer <=  1250) && (buffer >= -550)){
		ds.temperature[ds.channel] = (int16_t)buffer;
		return DS_READY;	
	}
	return DS_ERR;
}

//-------------------------------------
uint8_t DsGetType (uint8_t channel)
{
	uint8_t i;

	ds.type[ds.channel] = DS_TYPE_NOREAD;

	DsWriteByte(channel, DS_READ_ROM);				//read ROM
	i = DsReadByte(channel);

	if ((i == DS_TYPE_18B20) || (i == DS_TYPE_18S20))
		ds.type[ds.channel] = i;
	else
		ds.status[channel] = DS_ERR;

	return ds.type[ds.channel];
}

//-------------------------------------
uint8_t DsReset (uint8_t channel)
{
	uint8_t status = DS_ERR;

	ds.channel = channel;
	ONEW_SETO;
	ONEW_L;
	DsDelay(200);
	DsDelay(200);
	DsDelay(200);
	DsDelay(200);

		ONEW_SETI;
		ONEW_H;
		DsDelay(100);

		if (ONEW_GET == 0)
		{
			DsDelay(200);
			DsDelay(200);
			DsDelay(200);
			if (ONEW_GET > 0)
				status = DS_OK;
			else
				status = DS_SHORT;
		}
		else
			status = DS_NO_CONNECT;


	ds.status[channel] = status;
	return ds.status[channel];
}

//-------------------------------------
void DsWriteByte(uint8_t channel, uint8_t byte)
{
	uint8_t i;
	ds.channel = channel;

	for (i=0; i<8; i++)
	{
		DsWriteBit(byte & 0x01);
		byte >>= 1;
	}
}

//-------------------------------------
void DsWriteBit(uint8_t bit)
{
	ONEW_SETO;
	ONEW_L;
	DsDelay(6);

	if (bit > 0) ONEW_H;
	DsDelay(64);

	ONEW_H;
	DsDelay(20);
}

//-------------------------------------
uint8_t DsReadByte(uint8_t channel)
{
	uint8_t i;
	uint8_t temp = 0;
	ds.channel = channel;

	for (i=0; i<8; i++)
	{
		temp >>= 1;
		if (DsReadBit() > 0) temp |= 0x80;
	}
	return temp;
}

//-------------------------------------
uint8_t DsReadBit(void)
{
	uint8_t state = 0;

	ONEW_SETO;
	ONEW_L;
	DsDelay(6);

	ONEW_SETI;
	ONEW_H;

	DsDelay(6);
	if (ONEW_GET > 0) state = 1;

	DsDelay(78);
	return state;
}

